@@ -287,7 +287,7 @@ EXTER object sSAsystem_directoryA;
 EXTER char *kcl_self;
 #endif
 #if !defined(IN_MAIN) || !defined(ATT)
-EXTER bool initflag,raw_image;
+EXTER bool raw_image;
 #endif
 char *merge_system_directory();
 
@@ -564,14 +564,15 @@ EXTER object sLcompiled_function,sLpathname,sLcharacter,sLnumber,sLrational,sLfl
 EXTER object sLinteger,sLratio,sLshort_float,sLstandard_char,sLfixnum,sLpositive_fixnum, sLcomplex;
 EXTER object sLsingle_float,sLpackage,sLbignum,sLrandom_state,sLdouble_float,sLstream,sLbit,sLreadtable;
 EXTER object sLlong_float,sLhash_table,sLstructure,sLboolean;
+EXTER object sLdivision_by_zero,sLfloating_point_inexact,sLfloating_point_invalid_operation;
+EXTER object sLfloating_point_overflow,sLfloating_point_underflow;
 
 #ifdef ANSI_COMMON_LISP
 /* new ansi types */
 EXTER object sLarithmetic_error,sLbase_char,sLbase_string,sLbroadcast_stream,sLbuilt_in_class;
-EXTER object sLcell_error,sLclass,sLconcatenated_stream,sLcondition,sLcontrol_error,sLdivision_by_zero;
+EXTER object sLcell_error,sLclass,sLconcatenated_stream,sLcondition,sLcontrol_error;
 EXTER object sLecho_stream,sLend_of_file,sLerror,sLextended_char,sLfile_error,sLfile_stream;
-EXTER object sLfloating_point_inexact,sLfloating_point_invalid_operation,sLfloating_point_overflow;
-EXTER object sLfloating_point_underflow,sLgeneric_function,sLlogical_pathname,sLmethod,sLpackage_error;
+EXTER object sLgeneric_function,sLlogical_pathname,sLmethod,sLpackage_error;
 EXTER object sLparse_error,sLprint_not_readable,sLprogram_error,sLreader_error,sLserious_condition;
 EXTER object sLsimple_base_string,sLsimple_condition,sLsimple_type_error,sLsimple_warning,sLstandard_class;
 EXTER object sLstandard_generic_function,sLstandard_method,sLstandard_object,sLstorage_condition;
@@ -610,12 +611,6 @@ EXTER object TSor_pathname_string_symbol_stream;
 EXTER int interrupt_flag;		/* console interupt flag */
 EXTER int interrupt_enable;		/* console interupt enable */
 
-/*  CMPtemp  */
-EXTER object CMPtemp;
-EXTER object CMPtemp1;
-EXTER object CMPtemp2;
-EXTER object CMPtemp3;
-
 EXTER object sLAlink_arrayA;
 
 /* nfunlink.c */
@@ -629,12 +624,10 @@ object list_vector_new();
 object Iapply_ap();
 object IisFboundp();
 object IapplyVector();
-object c_apply_n();
 EXTER object sSPmemory;
 EXTER object sSPinit;
 object sLfset();
 object MakeAfun();
-extern  object Cstd_key_defaults[];   
 extern object call_proc0();
 /* extern object call_proc(); */
 /* extern object call_vproc(); */
