@@ -13,7 +13,7 @@
 #include <sys/mman.h>
 #define CLEAR_CACHE_LINE_SIZE 32
 #define CLEAR_CACHE {\
-    void *v1=memory->cfd.cfd_start,*v,*ve=v1+memory->cfd.cfd_size+CLEAR_CACHE_LINE_SIZE; \
+   void *v1=memory->cfd.cfd_start,*v,*ve=v1+memory->cfd.cfd_size;	\
    v1=(void *)((unsigned long)v1 & ~(CLEAR_CACHE_LINE_SIZE - 1));\
    for (v=v1;v<ve;v+=CLEAR_CACHE_LINE_SIZE) asm __volatile__ ("fdc 0(%0)" : : "r" (v) : "memory");\
    asm __volatile__ ("syncdma\n\tsync" : : "r" (v) : "memory");\
