@@ -95,7 +95,6 @@ extern DBEGIN_TY _stacktop, _stackbottom, _dbegin;
 #define NEED_TO_REINSTALL_SIGNALS
 
 /*#define HAVE_SIGACTION*/
-#define SETUP_SIG_STACK
 #define SV_ONSTACK 0
 #define SA_RESTART 0
 
@@ -117,7 +116,7 @@ extern DBEGIN_TY _stacktop, _stackbottom, _dbegin;
 /* use the slightly older unexec */
 #define UNIXSAVE "unexnt.c"
     
-#define RECREATE_HEAP if (initflag) { recreate_heap1(); \
+#define RECREATE_HEAP { recreate_heap1(); \
      terminal_io->sm.sm_object1->sm.sm_fp=stdout; \
      terminal_io->sm.sm_object0->sm.sm_fp=stdin; \
      init_shared_memory();}
@@ -141,7 +140,6 @@ extern DBEGIN_TY _stacktop, _stackbottom, _dbegin;
         ungetc(i, fp); \
     } while (0)
 		
-#define FCLOSE_SETBUF_OK 
 #define	IEEEFLOAT
 #define I386
 
@@ -221,3 +219,26 @@ extern int mingwlisten(FILE *);
 #define DBEGIN _dbegin
 
 #define NOFREE_ERR
+
+#define FPE_CODE(i_,v_) make_fixnum(fSfpe_code(FFN(fSfnstsw)(),FFN(fSstmxcsr)()))
+#define FPE_ADDR(i_,v_) make_fixnum(0)
+#define FPE_CTXT(v_) Cnil
+
+#define FPE_INIT Cnil
+
+#ifndef FE_INVALID
+#define FE_INVALID 1
+#define FE_DIVBYZERO 4
+#define FE_OVERFLOW 8
+#define FE_UNDERFLOW 16
+#define FE_INEXACT 32
+#endif
+
+#define FPE_FLTDIV 3
+#define FPE_FLTOVF 4
+#define FPE_FLTUND 5
+#define FPE_FLTRES 6
+#define FPE_FLTINV 7
+
+
+
