@@ -16,4 +16,16 @@
                            asm __volatile__ ("dcbst 0,%0\n\tsync\n\ticbi 0,%0\n\tsync\n\tisync": : "r" (v) : "memory");\
                         } while(0)
 
+#if SIZEOF_LONG == 4
 #define RELOC_H "elf32_ppc_reloc.h"
+#else
+#ifdef WORDS_BIGENDIAN
+#define RELOC_H "elf64_ppc_reloc.h"
+#define SPECIAL_RELOC_H "elf64_ppc_reloc_special.h"
+#else
+#define RELOC_H "elf64_ppcle_reloc.h"
+#define SPECIAL_RELOC_H "elf64_ppcle_reloc_special.h"
+#endif
+#define STATIC_FUNCTION_POINTERS
+#define C_GC_OFFSET 4
+#endif
