@@ -249,7 +249,6 @@ bool ovm_process_created;
 #endif
 
 
-bool saving_system;
 static int gc_time = -1;
 static int gc_start = 0;
 static int gc_recursive = 0;
@@ -268,7 +267,7 @@ struct apage {
   char apage_self[PAGESIZE];
 };
 
-long maxpage;
+/* long maxpage; */
 
 object sSAnotify_gbcA;
 
@@ -685,18 +684,10 @@ mark_object(object x) {
     case smm_probe:
       mark_object(x->sm.sm_object0);
       mark_object(x->sm.sm_object1);
-      if (saving_system)
-	{FILE *fp = x->sm.sm_fp;
-	if (fp != 0 /* && fp != stdin && fp !=stdout */
-	    )
-	  {fclose(fp);
-	  x->sm.sm_fp=0;
-	  }}
-      else
-	if (what_to_collect == t_contiguous &&
-	    x->sm.sm_fp &&
-	    x->sm.sm_buffer)
-	  mark_contblock(x->sm.sm_buffer, BUFSIZ);
+      if (what_to_collect == t_contiguous &&
+	  x->sm.sm_fp &&
+	  x->sm.sm_buffer)
+	mark_contblock(x->sm.sm_buffer, BUFSIZ);
       break;
       
     case smm_synonym:
@@ -1231,11 +1222,25 @@ GBC(enum type t) {
     (*GBC_enter_hook)();
   
   if (!GBC_enable)
-    error("GBC is not enabled");
+      error("GBC is not enabled");
   interrupt_enable = FALSE;
   
-  if (saving_system)
-    {t = t_relocatable; gc_time = -1;
+  if (saving_system) {
+
+    struct pageinfo *v;
+    void *x;
+    struct typemanager *tm=tm_of(t_stream);
+    unsigned j;
+
+    for (v=cell_list_head;v;v=v->next) 
+      if (tm->tm_type==v->type)
+	for (x=pagetochar(page(v)),j=tm->tm_nppage;j--;x+=tm->tm_size) {
+	  object o=x;
+	  if (type_of(o)==t_stream && !is_free(o) && o->sm.sm_fp && o->sm.sm_fp!=stdin && o->sm.sm_fp!=stdout)
+	    close_stream(o);
+	}
+
+    t = t_relocatable; gc_time = -1;
 #ifdef SGC
     if(sgc_enabled) sgc_quit();
 #endif    
@@ -1276,7 +1281,7 @@ GBC(enum type t) {
 #endif
   if (gc_time >=0 && !gc_recursive++) {gc_start=runtime();}
   
-  maxpage = page(heap_end);
+  /* maxpage = page(heap_end); */
   
   if (COLLECT_RELBLOCK_P) {
 
@@ -1362,7 +1367,7 @@ GBC(enum type t) {
     
     if (rb_start < rb_start1) {
       j = (rb_pointer-rb_start + PAGESIZE - 1)/PAGESIZE;
-      memcpy(rb_start,rb_start1,j*PAGESIZE);
+      memmove(rb_start,rb_start1,j*PAGESIZE);
     }
     
 #ifdef SGC
@@ -1503,6 +1508,7 @@ FFN(siLheap_report)(void) {
   vs_push(make_fixnum(0));
   vs_push(make_fixnum(0));
 #endif
+  vs_push(make_fixnum(phys_pages));
 
 }  
 
@@ -1567,7 +1573,7 @@ copy_relblock(char *p, int s)
  rb_pointer += s;
  rb_pointer1 += s;
  
- memcpy(q,p,s);
+ memmove(q,p,s);
  /* while (--s >= 0) */
  /*   { *q++ = *p++;} */
  
