@@ -27,6 +27,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <fcntl.h>
 
 static void
 init_main(void);
@@ -47,12 +48,10 @@ void initialize_process();
 #endif
 
 #include "include.h"
-#ifdef UNIX
 #include <signal.h>
-#endif
 #include "page.h"
 
-bool saving_system ;
+bool saving_system=FALSE;
 
 #ifdef BSD
 #include <sys/time.h>
@@ -77,9 +76,11 @@ char *system_directory;
 char stdin_buf[BUFSIZ + EXTRA_BUFSIZE];
 char stdout_buf[BUFSIZ + EXTRA_BUFSIZE];
 
+#include "stacks.h"
+
 int debug;			/* debug switch */
-int initflag = FALSE;		/* initialized flag */
-int raw_image = FALSE;		/* raw or saved image */
+int raw_image = TRUE;		/* raw or saved image -- CYGWIN will only place this in .data and not in .bss if initialized to non-zero */
+bool GBC_enable=FALSE;
 
 long real_maxpage;
 object sSAlisp_maxpagesA;
@@ -96,23 +97,15 @@ static object stack_space;
 #ifdef _WIN32
 unsigned int _dbegin = 0x10100000;
 #endif
-
-int cssize;
+#ifdef __CYGWIN__
+unsigned long _dbegin = 0;
+#endif
 
 #ifdef SGC
 int sgc_enabled;
 #endif
 void install_segmentation_catcher(void);
 
-#ifndef SIG_STACK_SIZE
-#define SIG_STACK_SIZE 1000
-#endif
-#ifndef SETUP_SIG_STACK
-#if defined(HAVE_SIGACTION) || defined(HAVE_SIGVEC)
-        struct sigstack estack;
-#endif
-#endif
-
 int
 cstack_dir(fixnum j) {
   static fixnum n;
@@ -146,11 +139,73 @@ mbrk(void *v) {
   return uc==(ufixnum)sbrk(uv-uc) ? 0 : -1;
 }
     
+#if defined(__CYGWIN__)||defined(__MINGW32__)
+
+#include <Windows.h>
+
+ufixnum
+get_phys_pages_no_malloc(void) {
+  MEMORYSTATUS m;
+
+  m.dwLength=sizeof(m);
+  GlobalMemoryStatus(&m);
+  return m.dwTotalPhys>>PAGEWIDTH;
+
+}
+
+#elif defined (DARWIN)
+
+#include <sys/sysctl.h>
+
+ufixnum
+get_phys_pages_no_malloc(void) {
+  uint64_t s;
+  size_t z=sizeof(s);
+  int m[2]={CTL_HW,HW_MEMSIZE};
+  
+  if (sysctl(m,2,&s,&z,NULL,0)==0)
+    return s>>PAGEWIDTH;
+
+  return 0;
+
+}
+
+#elif defined(__sun__)
+
+ufixnum
+get_phys_pages_no_malloc(void) {
+
+  return sysconf(_SC_PHYS_PAGES);
+
+}
+
+#else 
+
+ufixnum
+get_phys_pages_no_malloc(void) {
+  int l;
+  char b[PAGESIZE],*c;
+  const char *k="MemTotal:",*f="/proc/meminfo";
+  ufixnum res=0,n;
+  
+  if ((l=open(f,O_RDONLY))!=-1) {
+    if ((n=read(l,b,sizeof(b)))<sizeof(b) && 
+	!(b[n]=0) && 
+	(c=strstr(b,k)) && 
+	sscanf(c+strlen(k),"%lu",&n)==1)
+      res=n;
+    close(l);
+  }
+  return res>>(PAGEWIDTH-10);
+}
+
+#endif
+
 int
 update_real_maxpage(void) {
 
   ufixnum i,j,k;
-  void *end,*cur;
+  void *end,*cur,*beg;
 #ifdef __MINGW32__
   static fixnum n;
 
@@ -161,22 +216,22 @@ update_real_maxpage(void) {
 #endif
 
   massert(cur=sbrk(0));
+  beg=data_start ? data_start : cur;
   for (i=0,j=(1L<<log_maxpage_bound);j>PAGESIZE;j>>=1)
-    if ((end=data_start+i+j-PAGESIZE)>cur)
+    if ((end=beg+i+j-PAGESIZE)>cur)
       if (!mbrk(end)) {
 	real_maxpage=page(end);
 	i+=j;
       }
   massert(!mbrk(cur));
 
-#ifdef HAVE_SYSCONF_PHYS_PAGES
-  phys_pages=sysconf(_SC_PHYS_PAGES);
+  phys_pages=get_phys_pages_no_malloc();
+
 #ifdef BRK_DOES_NOT_GUARANTEE_ALLOCATION
-  if (phys_pages>0 && real_maxpage>phys_pages+first_data_page) real_maxpage=phys_pages+first_data_page;
-#endif
+  if (phys_pages>0 && real_maxpage>phys_pages+page(beg)) real_maxpage=phys_pages+page(beg);
 #endif
 
-  available_pages=real_maxpage-first_data_page;
+  available_pages=real_maxpage-page(beg);
   for (i=t_start,j=0;i<t_other;i++) {
     k=tm_table[i].tm_maxpage;
     if (tm_table[i].tm_type==t_relocatable)
@@ -238,7 +293,7 @@ minimize_image(void) {
   
 }
 
-DEFUN_NEW("SET-LOG-MAXPAGE-BOUND",fixnum,fSset_log_maxpage_bound,SI,1,1,NONE,II,OO,OO,OO,(fixnum l),"") {
+DEFUN_NEW("SET-LOG-MAXPAGE-BOUND",object,fSset_log_maxpage_bound,SI,1,1,NONE,II,OO,OO,OO,(fixnum l),"") {
 
   void *end,*dend;
   fixnum def=sizeof(fixnum)*8-1;
@@ -253,332 +308,159 @@ DEFUN_NEW("SET-LOG-MAXPAGE-BOUND",fixnum,fSset_log_maxpage_bound,SI,1,1,NONE,II,
     update_real_maxpage();
   }
 
-  return log_maxpage_bound;
+  return (object)log_maxpage_bound;
 
 }
 
+#ifdef NEED_STACK_CHK_GUARD
+
+unsigned long __stack_chk_guard=0;
+
+static unsigned long
+random_ulong() {
+ 
+  object y;
+  
+  vs_top=vs_base;
+  vs_push(Ct);
+  Lmake_random_state();
+  y=vs_pop;
+  vs_push(number_negate(find_symbol(make_simple_string("MOST-NEGATIVE-FIXNUM"),system_package)->s.s_dbind));
+  vs_push(y);
+  Lrandom();
+
+  return fixint(vs_pop);
+
+}
+#endif
+
+#ifdef HAVE_MPROTECT
+#include <sys/mman.h>
+int
+gcl_mprotect(void *v,unsigned long l,int p) {
+
+  int i;
+  char b[80];
+
+  if ((i=mprotect(v,l,p))) {
+    snprintf(b,sizeof(b),"mprotect failure: %p %lu %d\b",v,l,p);
+    perror(b);
+  }
+
+  return i;
+
+}
+#endif
 
 int
 main(int argc, char **argv, char **envp) {
-#ifdef BSD
-#ifdef RLIMIT_STACK
-	struct rlimit rl;
-#endif
-#endif
 
-#ifdef RECREATE_HEAP
-	RECREATE_HEAP
-#endif
-	
-#ifdef UNIX
-/*
-	if (argv[0][0] != '/')
-		error("can't get the program name");
-*/
-#ifdef GET_FULL_PATH_SELF
-	GET_FULL_PATH_SELF(kcl_self);
-#else
-	kcl_self = argv[0];
-#endif
-#ifdef FIX_FILENAME
-	{ int n = strlen(kcl_self);
-	 FIX_FILENAME(Cnil,kcl_self);
-	 if (strlen(kcl_self)> n) error("name grew");
-	}
-#endif	
-	*argv=kcl_self;
+  gcl_init_alloc(&argv);
 
 #ifdef CAN_UNRANDOMIZE_SBRK
 #include <stdio.h>
 #include <stdlib.h>
 #include "unrandomize.h"
 #endif
-
+  
 #ifdef LD_BIND_NOW
 #include <stdio.h>
 #include <stdlib.h>
 #include "ld_bind_now.h"
 #endif
-
-
-#if defined(DARWIN)
-	{
-	  extern void init_darwin_zone_compat ();
-	  init_darwin_zone_compat ();
-	}
-#endif
-
-        setbuf(stdin, stdin_buf); 
-	setbuf(stdout, stdout_buf);
+  
+  setbuf(stdin, stdin_buf); 
+  setbuf(stdout, stdout_buf);
 #ifdef _WIN32
-	_fmode = _O_BINARY;
-        _setmode( _fileno( stdin ), _O_BINARY );
-        _setmode( _fileno( stdout ), _O_BINARY );
-        _setmode( _fileno( stderr ), _O_BINARY );
+  _fmode = _O_BINARY;
+  _setmode( _fileno( stdin ), _O_BINARY );
+  _setmode( _fileno( stdout ), _O_BINARY );
+  _setmode( _fileno( stderr ), _O_BINARY );
 #endif
-	ARGC = argc;
-	ARGV = argv;
-#ifdef UNIX
-	ENVP = envp;
-#endif
-
-	if (!initflag) {
-
-		system_directory= (char *) malloc(strlen(argv[1])+3);
-					
-		strcpy(system_directory, argv[1]);
-		if (system_directory[0] != '/')
-			strcpy(system_directory, "./");
-		else {
-			int j;
-
-			for (j = strlen(system_directory);
-                             system_directory[j-1] != '/';  --j)
-				;
-			system_directory[j] = '\0';
-		}
-	}
-
-#endif
-#ifdef AOSVS
-
-
-
-
-
-
-
-
-
-
-
-
-#endif
-
-	if (!initflag && argc > 1) {
-#ifdef UNIX
-		if (argv[1][strlen(argv[1])-1] != '/')
-#endif
-#ifdef AOSVS
-
-#endif
-			error("can't get the system directory");
-		strcpy(system_directory, argv[1]);
-	}
-
-	GBC_enable = FALSE;
-
-	/* if stack_space not zero we have grown the stack space */
-	if (stack_space == 0)
-	  {
-	    vs_org = value_stack;
-	    vs_limit = &vs_org[VSSIZE];
-	    frs_org = frame_stack;
-	    frs_limit = &frs_org[FRSSIZE];
-	    bds_org = bind_stack;
-	    bds_limit = &bds_org[BDSSIZE];
-#ifdef KCLOVM
-	    bds_save_org = save_bind_stack;
-	    bds_save_top = bds_save_org - 1;
-	    bds_save_limit = &bds_save_org[BDSSIZE];
-#endif
-	    ihs_org = ihs_stack;
-	    ihs_limit = &ihs_org[IHSSIZE];}
-
-	vs_top = vs_base = vs_org;
-	clear_stack(vs_top,vs_top+200);
-	ihs_top = ihs_org-1;
-	bds_top = bds_org-1;
-	frs_top = frs_org-1;
-	cs_org = cs_base = &argc;
-#ifdef __ia64__
- {
-   extern void * __libc_ia64_register_backing_store_base;
-   cs_org2=cs_base2=__libc_ia64_register_backing_store_base;
- }
-#endif
-
-        cssize = (1L<<23);
-	install_segmentation_catcher();
-
-#ifdef BSD
-#ifdef RLIMIT_STACK
-	{
-	  unsigned long mss;
-	  mss=(real_maxpage/64)<<PAGEWIDTH;
-	  if (getrlimit(RLIMIT_STACK, &rl))
-	    error("Cannot get stack rlimit\n");
-	  if (rl.rlim_max != RLIM_INFINITY && rl.rlim_max < mss)
-	    mss=rl.rlim_max;
-	  if (rl.rlim_cur == RLIM_INFINITY || 
-	      rl.rlim_cur != mss) {
-	    rl.rlim_cur=mss;
-	    if (setrlimit(RLIMIT_STACK,&rl))
-	      fprintf(stderr,"Cannot set stack rlimit\n");/*FIXME work around make bug on mips*/
-	  }
-	  cssize = rl.rlim_cur/sizeof(*cs_org) - sizeof(*cs_org)*CSGETA;
-        }
-
-
-	/* Maybe the soft limit for data segment size is lower than the
-	 * hard limit.  In that case, we want as much as possible.
-	 */
-	if (getrlimit(RLIMIT_DATA, &rl))
-	    error("Cannot get data rlimit\n");
-	if (rl.rlim_cur != RLIM_INFINITY &&
-	    (rl.rlim_max == RLIM_INFINITY ||
-	     rl.rlim_max > rl.rlim_cur)) {
-	  rl.rlim_cur = rl.rlim_max;
-	  if (setrlimit(RLIMIT_DATA, &rl))
-	    error("Cannot set data rlimit\n");
-	}
-
-#endif	
-#endif
-
-#ifdef AV
-	cs_limit = cs_org - cssize;
-#endif
-#ifdef MV
-
-#endif
-        
-	set_maxpage();
-#ifdef SETUP_SIG_STACK
-	SETUP_SIG_STACK
+  ARGC = argc;
+  ARGV = argv;
+  ENVP = envp;
+  
+#ifdef GET_FULL_PATH_SELF
+  GET_FULL_PATH_SELF(kcl_self);
 #else
-#if defined(HAVE_SIGACTION) || defined(HAVE_SIGVEC)
-	  {
-	    /* make sure the stack is 8 byte aligned */
-	    static double estack_buf[32*SIGSTKSZ];
-	    static struct sigaltstack estack;
-	    
-	    bzero(estack_buf,sizeof(estack_buf));
-	    estack.ss_sp = estack_buf;
-	    estack.ss_flags = 0;                                   
-	    estack.ss_size = sizeof(estack_buf);                             
-	    if (sigaltstack(&estack, 0) < 0)                       
-	      error("sigaltstack");                             
-
-	  }
+  kcl_self = argv[0];
+#endif
+#ifdef __MINGW32__
+  {
+    char *s=kcl_self;
+    for (;*s;s++) if (*s=='\\') *s='/';
+  }
 #endif	
-#endif	
-	
-#ifdef SGC
-	memprotect_test_reset();
-#endif
-#ifdef GCL_GPROF
-	if (atexit(gprof_cleanup))
-	  error("Cannot setup gprof_cleanup on exit");
-#endif
-
-	if (initflag) {
-
-#ifdef _WIN32
-	  detect_wine();
-#endif
-
-#ifdef LD_BIND_NOW /*FIXME currently mips only, verify that these two
-		     requirements are the same*/
-	  reinit_gmp();
-#endif
-
-
-		if (saving_system) {
-
-			saving_system = FALSE;
-			terminal_io->sm.sm_object0->sm.sm_fp = stdin;
-			terminal_io->sm.sm_object1->sm.sm_fp = stdout;
-			gcl_init_big1();
-#ifdef INIT_CORE_END
-			INIT_CORE_END
-#endif			  
-		}
-
-		initflag = FALSE;
-		GBC_enable = TRUE;
-		vs_base = vs_top;
-		ihs_push(Cnil);
-		lex_new();
-		vs_base = vs_top;
-
-		interrupt_enable = TRUE;
-		install_default_signals();
-
-		sSAlisp_maxpagesA->s.s_dbind = make_fixnum(real_maxpage);
-		initflag = TRUE;
-#ifdef KCLOVM
-		ovm_user_context_change = change_contexts;
-		ovm_user_context_initialize = initialize_process;
-
-		v_init_processes();
-		ovm_process_created = 1;
+  *argv=kcl_self;
+  
+  vs_top = vs_base = vs_org;
+  ihs_top = ihs_org-1;
+  bds_top = bds_org-1;
+  frs_top = frs_org-1;
+
+  if (raw_image) {
+
+    printf("GCL (GNU Common Lisp)  %s  %ld pages\n",LISP_IMPLEMENTATION_VERSION,real_maxpage);
+    fflush(stdout);
+    
+    if (argc>1) {
+      massert(argv[1][strlen(argv[1])-1]=='/');
+      system_directory=argv[1];
+    }
+
+    initlisp();
+    lex_new();
+    
+    GBC_enable = TRUE;
+    
+    gcl_init_init();
+  
+    sLApackageA->s.s_dbind = user_package;
+    
+  } else {
+
+    terminal_io->sm.sm_object0->sm.sm_fp = stdin;
+    terminal_io->sm.sm_object1->sm.sm_fp = stdout;
+#ifdef LD_BIND_NOW /*FIXME currently mips only, verify that these two requirements are the same*/
+    reinit_gmp();
 #endif
+    gcl_init_big1();
 #ifdef HAVE_READLINE
-		gcl_init_readline_function();
+    gcl_init_readline_function();
+#endif
+#ifdef NEED_STACK_CHK_GUARD
+  __stack_chk_guard=random_ulong();/*Cannot be safely set inside a function which returns*/
 #endif
-	      again:
-		super_funcall(sStop_level);
-		if (type_of(sSAmultiply_stacksA->s.s_dbind)==t_fixnum) {
-                    multiply_stacks(fix(sSAmultiply_stacksA->s.s_dbind));
-                    goto  again;
-                }
-                    
-#ifdef USE_DLOPEN
-		unlink_loaded_files();
-#endif			
-		exit(0);
-	}
 
-	printf("GCL (GNU Common Lisp)  %s  %ld pages\n",
-	       LISP_IMPLEMENTATION_VERSION,
-	       real_maxpage);
-	fflush(stdout);
+  }
 
-	initlisp();
 #ifdef _WIN32
-	detect_wine();
+  detect_wine();
 #endif
 
-	vs_base = vs_top;
-	ihs_push(Cnil);
-	lex_new();
+  sSAlisp_maxpagesA->s.s_dbind = make_fixnum(real_maxpage);
 
-	GBC_enable = TRUE;
-
-	CMPtemp = CMPtemp1 = CMPtemp2 = CMPtemp3 = OBJNULL;
-
-#ifdef HAVE_LIBBFD
-	parse_plt();
-#endif
-	gcl_init_init();
-
-	sLApackageA->s.s_dbind = user_package;
-
-	lex_new();
-	vs_base = vs_top;
-	initflag = TRUE;
-
-	interrupt_enable = TRUE;
-
-	raw_image=TRUE;
-
-	super_funcall(sStop_level);
-
-	return 0;
+  ihs_push(Cnil);
+  lex_new();
+  vs_base = vs_top;
+  
+  interrupt_enable = TRUE;
+  install_default_signals();
+    
+  do 
+    super_funcall(sStop_level);
+  while (type_of(sSAmultiply_stacksA->s.s_dbind)==t_fixnum && multiply_stacks(fix(sSAmultiply_stacksA->s.s_dbind)));
+    
+  return 0;
 
 }
 
 /* catch certain signals */
 void install_segmentation_catcher(void)
 {
-#ifdef INSTALL_SEGMENTATION_CATCHER
-  INSTALL_SEGMENTATION_CATCHER;
-#else
-#ifdef SIGSEGV
-       (void) gcl_signal(SIGSEGV,segmentation_catcher);
-#endif
-#endif
+  (void) gcl_signal(SIGSEGV,segmentation_catcher);
+  (void) gcl_signal(SIGBUS,segmentation_catcher);
 }
 
 int catch_fatal=1;
@@ -596,16 +478,14 @@ error(char *s)
 	   FEerror("Caught fatal error [memory may be damaged]",0); }
 	printf("\nUnrecoverable error: %s.\n", s);
 	fflush(stdout);
-#ifdef UNIX
 	abort();
-#endif
 }
 
 static void
 initlisp(void) {
 
         void *v=&v;
-
+	
 	if (NULL_OR_ON_C_STACK(v) == 0
 #if defined(IM_FIX_BASE)
              || NULL_OR_ON_C_STACK(IM_FIX_BASE) == 0
@@ -630,8 +510,6 @@ initlisp(void) {
 	  error("NULL_OR_ON_C_STACK macro invalid");
 	}
 	
-	gcl_init_alloc();
-	
 	Cnil->fw=0;
 	set_type_of(Cnil,t_symbol);
  	Cnil->c.c_cdr=Cnil;
@@ -699,17 +577,9 @@ initlisp(void) {
 	gcl_init_print();
 	gcl_init_GBC();
 
-#if defined ( UNIX ) || defined ( __MINGW32__ )
-#ifndef DGUX
 	gcl_init_unixfasl();
 	gcl_init_unixsys();
 	gcl_init_unixsave();
-#else
-
-
-
-#endif
-#endif
 
 	gcl_init_alloc_function();
 	gcl_init_array_function();
@@ -732,10 +602,8 @@ initlisp(void) {
 	gcl_init_hash();
 	gcl_init_cfun();
 
-#ifdef UNIX
 	gcl_init_unixfsys();
 	gcl_init_unixtime();
-#endif
 	gcl_init_eval();
 	gcl_init_lex();
 	gcl_init_prog();
@@ -764,6 +632,9 @@ initlisp(void) {
 #ifdef CMAC
 	gcl_init_cmac();
 #endif	
+#ifdef HAVE_READLINE
+	gcl_init_readline();
+#endif
 
 }
 object
@@ -806,13 +677,6 @@ ihs_overflow(void) {
 
 void
 segmentation_catcher(int i) {
-#ifndef SIG_STACK_SIZE 
-  int x;
-  if (&x < cs_limit)
-    cs_overflow();
-  else 
-    printf("Segmentation violation: c stack ok:signalling error");
-#endif
   error("Segmentation violation.");
 }
 
@@ -837,23 +701,19 @@ segmentation_catcher(int i) {
 /* } */
 
 DEFUNO_NEW("BYE",object,fLbye,LISP
-       ,0,1,NONE,OI,OO,OO,OO,void,Lby,(fixnum exitc),"")
+       ,0,1,NONE,OO,OO,OO,OO,void,Lby,(object exitc),"")
 {	int n=VFUN_NARGS;
 	int exit_code;
-	if (n>=1) exit_code=exitc;else exit_code=0;
+	if (n>=1) exit_code=fix(exitc);else exit_code=0;
 
-#ifdef UNIX
 /*	printf("Bye.\n"); */
 	exit(exit_code);
-#else
-	RETURN(1,int,exit_code, 0); 
-#endif
 
 }
 
 
 DEFUN_NEW("QUIT",object,fLquit,LISP
-       ,0,1,NONE,OI,OO,OO,OO,(fixnum exitc),"")
+       ,0,1,NONE,OO,OO,OO,OO,(object exitc),"")
 {	return FFN(fLbye)(exitc); }
  
 /* DEFUN_NEW("EXIT",object,fLexit,LISP */
@@ -887,7 +747,6 @@ FFN(siLargv)(void) {
 
 }
 
-#ifdef UNIX
 static void
 FFN(siLgetenv)(void) {
 
@@ -914,7 +773,6 @@ FFN(siLgetenv)(void) {
     vs_base[0] = Cnil;
 
 }
-#endif
 
 object *vs_marker;
 
@@ -941,7 +799,7 @@ FFN(siLcatch_fatal)(int i) {
 
 LFD(siLreset_stack_limits)(void)
 {
-  int i=0;
+  long i=0;
 
   check_arg(0);
   if(catch_fatal <0) catch_fatal=1;
@@ -1109,8 +967,9 @@ FFN(siLsave_system)(void) {
 
   minimize_image();
 
-  Lsave();
   saving_system = FALSE;
+
+  Lsave();
   alloc_page(-(holepage+nrbpage));
 
 }
@@ -1128,9 +987,7 @@ init_main(void) {
   make_si_function("ARGC", siLargc);
   make_si_function("ARGV", siLargv);
   
-#ifdef UNIX
   make_si_function("GETENV", siLgetenv);
-#endif
   
   make_si_function("MARK-VS", siLmark_vs);
   make_si_function("CHECK-VS", siLcheck_vs);
@@ -1165,11 +1022,9 @@ init_main(void) {
   ADD_FEATURE("GPROF");
 #endif	 
   
-#ifdef UNIX
 #ifndef _WIN32	 
   ADD_FEATURE("UNIX");
 #endif	
-#endif
 
 #ifdef _WIN32
   ADD_FEATURE("WINNT");
@@ -1249,3 +1104,98 @@ init_main(void) {
 #ifdef SGC
 #include "writable.h"
 #endif
+
+#ifdef HAVE_PRINT_INSN_I386
+
+#include "dis-asm.h"
+
+static char b[4096],*bp;
+
+static int
+my_fprintf(void *v,const char *f,...) {
+  va_list va;
+  int r;
+  va_start(va,f);
+  bp+=(r=vsnprintf(bp,sizeof(b)-(bp-b),f,va));
+  va_end(va);
+  return r;
+}
+
+static int
+my_read(bfd_vma memaddr, bfd_byte *myaddr, unsigned int length, struct disassemble_info *dinfo) {
+  memcpy(myaddr,(void *)(long)memaddr,length);
+  return 0;
+}
+
+static void
+my_pa(bfd_vma addr,struct disassemble_info *dinfo) {
+  dinfo->fprintf_func(dinfo->stream,"%p",(void *)(long)addr);
+}
+
+#endif
+
+DEFUN_NEW("DISASSEMBLE-INSTRUCTION",object,fSdisassemble_instruction,SI,1,1,NONE,OI,OO,OO,OO,(fixnum addr),"") {
+
+#ifdef HAVE_PRINT_INSN_I386
+
+  static disassemble_info i;
+  void *v;
+  int (*s)();
+  int j;
+
+  memset(&i,0,sizeof(i));
+#ifdef __i386__
+  i.disassembler_options="i386";
+#endif
+  i.fprintf_func=my_fprintf;
+  i.read_memory_func=my_read;
+  i.print_address_func=my_pa;
+  bp=b;
+
+  if ((v=dlopen("libopcodes.so",RTLD_NOW))) {
+    if ((s=dlsym(v,"print_insn_i386"))) {
+      j=s(addr,&i);
+      my_fprintf(NULL," ;");
+      return MMcons(make_simple_string(b),make_fixnum(j));
+    }
+    massert(!dlclose(v));
+  }
+
+#endif
+
+  return MMcons(make_simple_string("fnop ;"),make_fixnum(0));
+
+}
+
+typedef struct {
+  enum type tt;
+  struct typemanager *tp;
+} Tbl;
+
+#define Tblof(a_)       {(a_),tm_of(a_)}
+#define tblookup(a_,b_) ({Tbl *tb=tb1;(b_)=(a_);for (;tb->tt && tb->b_!=(b_);tb++);tb->tt;})
+#define mtm_of(a_)      (a_)>=t_other ? NULL : tm_of(a_)
+
+DEFUN_NEW("FUNCTION-BY-ADDRESS",object,fSfunction_by_address,SI,1,1,NONE,OI,OO,OO,OO,(fixnum ad),"") {
+
+  ufixnum m=-1,mm,j;
+  void *o;
+  object x,xx=Cnil;
+  Tbl tb1[]={Tblof(t_sfun),Tblof(t_cfun),Tblof(t_vfun),Tblof(t_afun),Tblof(t_gfun),Tblof(t_closure),Tblof(t_cclosure),{0}};
+  struct typemanager *tp;
+  enum type tt;
+  struct pageinfo *v;
+
+  if (VALID_DATA_ADDRESS_P(ad))
+    for (v=cell_list_head;v;v=v->next)
+      if (tblookup(mtm_of(v->type),tp))
+	for (o=pagetochar(page(v)),j=tp->tm_nppage;j--;o+=tp->tm_size)
+	  if (tblookup(type_of((x=o)),tt))
+	    if (!is_free(x) && (mm=ad-(ufixnum)x->sfn.sfn_self)<m) {
+	      m=mm;
+	      xx=x;
+	    }
+  
+  return xx;
+
+}
