@@ -48,6 +48,7 @@ License for more details.
 #define  ELF_R_INFO(a,b)       Mjoin(ELF,_R_INFO)(a,b)
 #define ELF_ST_BIND(a)         Mjoin(ELF,_ST_BIND)(a)
 #define ELF_ST_TYPE(a)         Mjoin(ELF,_ST_TYPE)(a)
+#define ELF_ST_INFO(a,b)       Mjoin(ELF,_ST_INFO)(a,b)
 #define ELF_ST_VISIBILITY(a)   Mjoin(ELF,_ST_VISIBILITY)(a)
 
 
@@ -90,6 +91,8 @@ ovchku(ul v,ul m) {
 
 }
 
+static char *init_section_name=".text"; 
+
 #ifdef SPECIAL_RELOC_H
 #include SPECIAL_RELOC_H
 #endif
@@ -197,7 +200,7 @@ find_init_address(Sym *sym,Sym *syme,Shdr *sec1,Shdr *sece,
     if (sec<sec1 || sec>=sece)
       continue;
 
-    if (strcmp(sn+sec->sh_name,".text"))
+    if (strcmp(sn+sec->sh_name,init_section_name))
       continue;
 
     if (memcmp("init_",st1+sym->st_name,4))
@@ -228,6 +231,9 @@ relocate_symbols(Sym *sym,Sym *syme,Shdr *sec1,Shdr *sece,const char *st1) {
 
     else if ((a=find_sym_ptable(st1+sym->st_name)))
       sym->st_value=a->address;
+
+    else if (ELF_ST_BIND(sym->st_info)!=STB_LOCAL)
+      massert(!fprintf(stderr,"Unrelocated non-local symbol: %s\n",st1+sym->st_name));
 	
   }
 
@@ -354,7 +360,7 @@ parse_map(void *v1,Shdr **sec1,Shdr **sece,
     *dst1=v1+sec->sh_offset;
   }
 
-  for (*end=0,sec=*sec1;sec<*sece;sec++) 
+  for (*end=fhp->e_shoff+fhp->e_shnum*fhp->e_shentsize,sec=*sec1;sec<*sece;sec++) 
     *end=ulmax(*end,sec->sh_offset+sec->sh_size);
 
   return 0;
@@ -521,24 +527,17 @@ seek_to_end_ofile(FILE *fp) {
 
 }
 
-#ifdef HAVE_BUILTIN_CLEAR_CACHE
 static int
 clear_protect_memory(object memory) {
 
   void *p,*pe;
-  int i;
 
   p=(void *)((unsigned long)memory->cfd.cfd_start & ~(PAGESIZE-1));
   pe=(void *)((unsigned long)(memory->cfd.cfd_start+memory->cfd.cfd_size + PAGESIZE-1) & ~(PAGESIZE-1));
 
-  i=mprotect(p,pe-p,PROT_READ|PROT_WRITE|PROT_EXEC);
-
-  __builtin___clear_cache((void *)memory->cfd.cfd_start,(void *)memory->cfd.cfd_start+memory->cfd.cfd_size);
-
-  return i;
+  return gcl_mprotect(p,pe-p,PROT_READ|PROT_WRITE|PROT_EXEC);
 
 }
-#endif
 
 int
 fasload(object faslfile) {
@@ -560,7 +559,7 @@ fasload(object faslfile) {
   massert(!parse_map(v1,&sec1,&sece,&sn,&sym1,&syme,&st1,&end,&dsym1,&dsyme,&dst1));
   
 #ifdef SPECIAL_RELOC_H
-  massert(!label_got_symbols(v1,sec1,sece,sym1,syme,st1,got));
+  massert(!label_got_symbols(v1,sec1,sece,sym1,syme,st1,sn,got));
 #endif
 
   massert(memory=load_memory(sec1,sece,v1,&got,&gote));
@@ -577,12 +576,12 @@ fasload(object faslfile) {
   massert(!un_mmap(v1,ve));
   close_stream(faslfile);
   
-#ifdef HAVE_BUILTIN_CLEAR_CACHE
   massert(!clear_protect_memory(memory));
-#else
-#ifdef CLEAR_CACHE
+
+#if defined(HAVE_BUILTIN_CLEAR_CACHE)
+  __builtin___clear_cache((void *)memory->cfd.cfd_start,(void *)memory->cfd.cfd_start+memory->cfd.cfd_size);
+#elif defined(CLEAR_CACHE)
   CLEAR_CACHE;
-#endif
 #endif  
 
   init_address-=(ul)memory->cfd.cfd_start;
