@@ -10,10 +10,6 @@
 static void
 sgc_mark_object1(object);
 
-static void
-sgc_mprotect(long, long, int);
-
-
 #ifdef BSD
 /* ulong may have been defined in mp.h but the define is no longer needed */
 #undef ulong
@@ -413,18 +409,10 @@ sgc_mark_object1(object x) {
     case smm_probe:
       sgc_mark_object(x->sm.sm_object0);
       sgc_mark_object(x->sm.sm_object1);
-      if (saving_system) {
-	FILE *fp = x->sm.sm_fp;
-	if (fp != 0 && fp != stdin && fp !=stdout) {
-	  fclose(fp);
-	  x->sm.sm_fp=0;
-	}
-      }
-      else
-	if (what_to_collect == t_contiguous &&
-	    x->sm.sm_fp &&
-	    x->sm.sm_buffer)
-	  mark_contblock(x->sm.sm_buffer, BUFSIZ);
+      if (what_to_collect == t_contiguous &&
+	  x->sm.sm_fp &&
+	  x->sm.sm_buffer)
+	mark_contblock(x->sm.sm_buffer, BUFSIZ);
       break;
       
     case smm_synonym:
@@ -916,7 +904,7 @@ memprotect_handler_test(int sig, long code, void *scp, char *addr) {
     memprotect_result=memprotect_bad_fault_address;
   else
     memprotect_result=memprotect_none;
-  mprotect(memprotect_test_address,PAGESIZE,PROT_READ_WRITE_EXEC);
+  gcl_mprotect(memprotect_test_address,PAGESIZE,PROT_READ_WRITE_EXEC);
 
 }
 
@@ -961,7 +949,7 @@ memprotect_test(void) {
   { /* mips kernel bug test -- SIGBUS with no faddr when floating point is emulated. */
     float *f1=(void *)memprotect_test_address,*f2=(void *)b2,*f1e=f1+p/sizeof(*f1);
   
-    if (mprotect(memprotect_test_address,p,PROT_READ_EXEC)) {
+    if (gcl_mprotect(memprotect_test_address,p,PROT_READ_EXEC)) {
       memprotect_result=memprotect_cannot_protect;
       return -1;
     }
@@ -977,7 +965,7 @@ memprotect_test(void) {
     memprotect_handler_invocations=0;
 
   }
-  if (mprotect(memprotect_test_address,p,PROT_READ_EXEC)) {
+  if (gcl_mprotect(memprotect_test_address,p,PROT_READ_EXEC)) {
     memprotect_result=memprotect_cannot_protect;
     return -1;
   }
@@ -1305,21 +1293,10 @@ sgc_start(void) {
     writable_pages=0;
 
     for (pi=cell_list_head;pi;pi=pi->next) {
-      void *v,*ve;
       if (pi->sgc_flags&SGC_WRITABLE)
 	SET_WRITABLE(page(pi));
       else
 	tm_of(pi->type)->tm_alt_npage++;
-#ifndef NO_SETBUF /*FIXME, implement restartable getc with read in readc_stream*/
-      if (pi->type!=(tm=tm_of(t_stream))->tm_type) continue;
-      for (v=pagetochar(page(pi)),ve=v+tm->tm_nppage*tm->tm_size;v<ve;v+=tm->tm_size) {
-	object x=v;
-	if (type_of(x)!=t_stream || is_free(x)) continue;
-	if (x->sm.sm_buffer) 
-	  for (i=page(x->sm.sm_buffer);i<=page(x->sm.sm_buffer+BUFSIZ-1);i++)
-	    SET_WRITABLE(i);
-      }
-#endif
     }
     for (pi=contblock_list_head;pi;pi=pi->next)/*FIXME*/
       if (pi->sgc_flags&SGC_WRITABLE)
@@ -1327,14 +1304,16 @@ sgc_start(void) {
 	  SET_WRITABLE(page(pi)+i);
       else
 	tm_of(t_contiguous)->tm_alt_npage+=pi->in_use;
-#ifdef GCL_GPROF
     {
-      extern object gprof_array;
-      if (gprof_array!=Cnil)
-	for (i=0;i<(gprof_array->st.st_fillp +PAGESIZE-1)/PAGESIZE;i++)
-	  SET_WRITABLE(page(gprof_array->st.st_self)+i);
+      extern object malloc_list;
+      object x;
+
+      for (x=malloc_list;x!=Cnil;x=x->c.c_cdr)
+	if (x->c.c_car->st.st_adjustable)
+	  for (i=page(x->c.c_car->st.st_self);i<=page(x->c.c_car->st.st_self+x->c.c_car->st.st_fillp-1);i++)
+	    SET_WRITABLE(i);
     }
-#endif
+
     for (i=page(heap_end);i<page(old_rb_start);i++)
 	SET_WRITABLE(i);
     tm_of(t_relocatable)->tm_alt_npage=page(rb_start)-page(old_rb_start);
@@ -1349,8 +1328,9 @@ sgc_start(void) {
      and modified the tm_table;
      Turn  memory protection on for the pages which are writable.
   */
-  memory_protect(1);
   sgc_enabled=1;
+  if (memory_protect(1)) 
+    sgc_quit();
   if (sSAnotify_gbcA->s.s_dbind != Cnil) {
     printf("[SGC on]"); 
     fflush(stdout);
@@ -1496,8 +1476,10 @@ memprotect_handler(int sig, long code, void *scp, char *addr) {
   printf("fault:0x%x [%d] (%d)  ",faddr,page(faddr),faddr >= core_end);
 #endif 
   if (faddr >= (void *)core_end || faddr < data_start) {
-    if (fault_count > 300) error("fault count too high");
-    fault_count ++;
+    static void *old_faddr;
+    if (old_faddr==faddr) 
+      if (fault_count++ > 300) error("fault count too high");
+    old_faddr=faddr;
     INSTALL_MPROTECT_HANDLER;
     return;
   }
@@ -1520,7 +1502,7 @@ memprotect_handler(int sig, long code, void *scp, char *addr) {
     INSTALL_MPROTECT_HANDLER;
 #endif
 
-    mprotect(pagetoinfo(p),PAGESIZE,PROT_READ_WRITE_EXEC);
+    massert(!gcl_mprotect(pagetoinfo(p),PAGESIZE,PROT_READ_WRITE_EXEC));
     SET_WRITABLE(p);
     fault_pages++;
 
@@ -1536,7 +1518,7 @@ memprotect_handler(int sig, long code, void *scp, char *addr) {
 
 }
 
-static void
+static int
 sgc_mprotect(long pbeg, long n, int writable) {
   /* CHECK_RANGE(pbeg,n);  */
 #ifdef DEBUG_MPROTECT
@@ -1546,14 +1528,18 @@ sgc_mprotect(long pbeg, long n, int writable) {
 	 pagetoinfo(pbeg), n * PAGESIZE, sbrk(0));
   fflush(stdout);
 #endif  
-  if(mprotect(pagetoinfo(pbeg),n*PAGESIZE,
-	      (writable & SGC_WRITABLE ? PROT_READ_WRITE_EXEC : PROT_READ_EXEC)))
-    FEerror("Couldn't protect",0);
+  if(gcl_mprotect(pagetoinfo(pbeg),n*PAGESIZE,(writable & SGC_WRITABLE ? PROT_READ_WRITE_EXEC : PROT_READ_EXEC))) {
+    perror("sgc disabled");
+    return -1;
+  }
+
+  return 0;
+
 }
 
 
 
-void
+int
 memory_protect(int on) {
 
   unsigned long i,beg,end= page(core_end);
@@ -1567,7 +1553,7 @@ memory_protect(int on) {
   if (on==0) {
     sgc_mprotect(first_protectable_page,end-first_protectable_page,SGC_WRITABLE);
     install_segmentation_catcher();
-    return;
+    return 0;
   }
 
   INSTALL_MPROTECT_HANDLER;
@@ -1578,12 +1564,15 @@ memory_protect(int on) {
 
     if (writable==IS_WRITABLE(i) && i<=end) continue;
 
-    sgc_mprotect(beg,i-beg,writable);
+    if (sgc_mprotect(beg,i-beg,writable)) 
+      return -1;
     writable=1-writable;
     beg=i;
 
   }
 
+  return 0;
+
 }
 
 static void
